--
-- ProfiLua
--
-- Example #1
--
--
-- NOTE: Code uses TABs! (indent = 2)
--
-- 
-- 11/2010
-- (c)ASkr, www.askrprojects.net
--
--



-- - THE FILENAME <n.lua> NEEDS TO MATCH THE NUMBER OF THE DLL INSIDE PROFILAB!
--   For DLL1 -> 1.lua
--       DLL2 -> 2.lua
--       ...  -> ...
-- - THE FILE HAS TO BE AVAILABLE BEFORE THE DLL IS IMPORTED INTO PROFILAB!
--   At least the number of in- and output channels and their names need to be present.




-- REQUIRED:
-- Number of (user) in- and outputs
--
-- NOTES:
--
--   - One extra INPUT ("CONSOLE") is automatically added at first position:
--     If this pin is set to a high logic state (>2.5, the Profilab convention)
--     at program start, a debug text console is opened. It may be used for debugging.
--     All Lua "print", "io.write", etc... outputs
--     will be redirected to this window.
--     NOTE: Do not close this window while ProfiLab is running.
--
--   - One extra OUTPUT ("ERROR") is automatically added at first position:
--     total channels = gNumOutputs + 1
--     This output is an ERROR pin. If it is set, something went wrong...
--     Additionally, this channel is hidden inside the lua environment.
--     First "user" channel (both input and output) start at "1".
--
-- *****************************************************************************************
-- *** CHANGING THE AMOUNT OF IN- OR OUTPUTS REQUIRES RELOADING THE DLL INSIDE PROFILAB! ***
-- *** Even if the updates are already visible! Double click your DLL and choose IMPORT. ***
-- *****************************************************************************************
--      
gNumInputs  = 4
gNumOutputs = 5



-- REQUIRED:
-- Names of (user) in- and outputs (used in "gInputName()" and "gOutputName()").
--
-- NOTES:
--
--   - For string arguments, the name of the channel needs to be preceeded with an "$".
--     E.g: "Kekse"  -> a number
--          "$Kekse" -> a string
--
--   - Do not use two strings in one row for in- and output!
--     Only one string per row is allowed.
--                +------------+
--               -|Num1      OA|-
--               -|$Str1     OB|-   <- OK
--               -|$Str2    $OC|-   <- ERROR, two strings in a row
--               -|Num2     $OD|-   <- OK
--                +------------+
--
--     Sometimes, this requires some dummy lines.
--     E.g.: Three input- and three output strings:
--                +------------+
--               -|$In1   ODum1|-
--               -|$In2   ODum2|-
--               -|$In3   ODum3|-
--               -|IDum1  $out1|-
--               -|IDum2  $out2|-
--               -|IDum3  $out3|-
--                +------------+
--
--   - For Lua newbies:
--        Standard (non hashed) Lua tables (comparable to an "C-array") begin with index "1" (not "0")
--        E.g.: Elements from this (non-key) table
--          mytable = {5, 6, 7, "Nase"}
--        are:
--          mytable[1] == 5
--          mytable[2] == 6
--          mytable[3] == 7
--          mytable[4] == "Nase"
--          mytable[5] == nil
--
--
-- *****************************************************************************************
-- *** CHANGING THE NAMES OF IN- OR OUTPUTS REQUIRES RELOADING THE DLL INSIDE PROFILAB!  ***
-- *** Even if the updates are already visible! Double click your DLL and choose IMPORT. ***
-- *****************************************************************************************
--
InputNames  = {"Num1","Num2","$Str1","$Str2"}
OutputNames = {"Add", "Sub", "Mul",  "Dummy", "$StrOut"}




------------------------------------------------------------------------------
-- gInputName (ALWAYS REQUIRED)
--
-- No changes required if you use the "InputNames"-table above.
------------------------------------------------------------------------------
function gInputName(channel)
	return InputNames[channel]
end




------------------------------------------------------------------------------
-- gOutputName (ALWAYS REQUIRED)
--
-- No changes required if you use the <OutputNames>-table above.
------------------------------------------------------------------------------
function gOutputName(channel)
	return OutputNames[channel]
end



------------------------------------------------------------------------------
-- gSimStart (ALWAYS REQUIRED)
--
-- <In>, a table of length <gNumInput>, containing *ONLY* numerical arguments!
-- String inputs are set to a dummy "0" (a number, not a string). Ignore them!
-- 
-- The return argument has to be a table of length <gNumOutput>.
-- Although only numerical channels can be initialized, *ALL* channels,
-- including strings, need to be present in the return table.
-- The value of the strings do not matter (can even be numerical).
------------------------------------------------------------------------------
function gSimStart(In)
	-- return a table with initial values for channels
	-- "Add", "Sub", "Mul", "Dummy" and "$StrOut"
	return {0, 0, 0, 666, "DUMMY"}
end



------------------------------------------------------------------------------
-- gSimStop (ALWAYS REQUIRED)
--
-- You usually won't need this...
-- Lua's garbage collector is smart enough to destroy all the nice things
-- you created before...
--
-- DO NOT RETURN ANY VALUE!
------------------------------------------------------------------------------
function gSimStop(In)
end



------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- gCalculate (ALWAYS REQUIRED)
--
-- <In>, a table of length <gNumInput>, containing *ALL* arguments!
-- Numbers and strings mixed, as specified by <InputNames>.
-- 
-- The return argument has to be a table of length <gNumOutput>.
-- Take care which of the channels require a number and which one needs a string!
-- Order is the same as specified by <OutputNames>
------------------------------------------------------------------------------
------------------------------------------------------------------------------
function gCalculate(In)

	-- This will only "print" if the debug console is activated.
	-- Tip: Execution can be halted by pressing "PAUSE" key (in debug console).
	--      Continue with "SPACE".
	print("Hello ProfiLua "..PROFILUAVERSION..", DLL#"..tostring(gPUser[101]), In[1], In[2], In[3], In[4])

	-- return a table with calculated values for channel "Add", "Sub" and "Mul"
	return {In[1]+In[2], In[1]-In[2], In[1]*In[2], 666, In[3].."<-MID->"..In[4]}

end


